<?php

namespace Ardent\UtilBundle\EntityBuilder;

use Doctrine\ORM\EntityManagerInterface;
use Exception;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ChoiceQuestion;
use Symfony\Component\Console\Question\ConfirmationQuestion;
use Symfony\Component\Console\Style\SymfonyStyle;

abstract class AbstractEntityCommand extends Command
{
    public function __construct(
        private EntityManagerInterface $em,
    )
    {
        parent::__construct();
    }

    abstract function getConfig(Builder $builder): void;

    abstract function getEntityClass(): string;

    protected function configure(): void
    {
        $this
            ->addArgument('id', InputArgument::OPTIONAL, 'Object Id')
            ->addOption('create', 'c', InputOption::VALUE_NONE, 'Create entity')
            ->addOption('modify', 'm', InputOption::VALUE_NONE, 'Edit entity')
            ->addOption('delete', 'd', InputOption::VALUE_NONE, 'Delete entity')
            ->addOption('show', 's', InputOption::VALUE_NONE, 'Show entity');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $id = $input->getArgument('id');

        $builder = new Builder($io);
        $this->getConfig($builder);

        if ($input->getOption('create')) {
            do {
                $io->info(sprintf('Creating new "%s"', $this->getEntityClass()));
                $entity = new ($this->getEntityClass())();
                $builder->setAll($entity);
                $this->em->persist($entity);

                if ($io->askQuestion(new ConfirmationQuestion(sprintf('Are you sure you want to create %s with above properties', $entity)))) {
                    $io->success(sprintf('Object "%s" (id: %s) stored successfully', $entity, $entity->getId()));
                }
            } while ($io->askQuestion(new ConfirmationQuestion('Create another?', false)));
        } elseif ($input->getOption('modify')) {
            $io->info(sprintf('Modifying "%s"', $this->getEntityClass()));
            $entity = $this->select($io, $id);
            $builder->setByQuestion($entity);
            $this->em->persist($entity);
        } elseif ($input->getOption('delete')) {
            $io->info(sprintf('Deleting "%s"', $this->getEntityClass()));
            $entity = $this->select($io, $id);
            if ($io->askQuestion(new ConfirmationQuestion(sprintf('Are you sure you want to delete %s', $entity)))) {
                $name = (string)$entity;
                $this->em->remove($entity);
                $io->success(sprintf('Config "%s" deleted', $name));
            }
        } elseif ($input->getOption('show')) {
            $io->info(sprintf('Showing "%s"', $this->getEntityClass()));
            $entity = $this->select($io, $id);
            $table = $io->createTable();
            $table->setHeaders(['property', 'value']);
            $table->setRows($builder->getAllProperties($entity));
            $table->setHeaderTitle(sprintf('id: %s', $entity->getId()));
            $table->render();
        } else {
            $io->error('Select at least one of the options: Create (-c), Modify (-m), Delete (-d) or Show (-s)');
            return Command::INVALID;
        }

        $this->em->flush();

        return Command::SUCCESS;
    }

    private function select(SymfonyStyle $io, ?int $id): object
    {
        $repo = $this->em->getRepository($this->getEntityClass());
        if ($id) {
            $object = $repo->find($id);
            if ($object) {
                return $object;
            }
        }
        $entities = $repo->findAll();

        if (empty($entities)) {
            throw new Exception('No objects found');
        }
        $question = new ChoiceQuestion('Select configuration to delete', $entities);
        return $io->askQuestion($question);
    }
}